// MIT © 2017 azu
"use strict";
import { MapLike } from "map-like";
var flagMap = new MapLike();
/**
 * IT IS FOR TESTING
 */
export var resetFlags = function () {
    flagMap.clear();
};
/**
 * set feature flag
 * @param {string} flagName
 * @param {boolean} status
 */
export var setFeature = function (flagName, status) {
    flagMap.set(flagName, status);
};
/**
 * If the feature flag of `flagName` is enabled, return true.
 * @param {string} flagName
 * @param {boolean=false} [loose]
 * @returns {boolean}
 */
export var isFeatureEnabled = function (flagName, _a) {
    var _b = (_a === void 0 ? {} : _a).loose, loose = _b === void 0 ? false : _b;
    if (!flagMap.has(flagName)) {
        if (loose) {
            // loose-mode, return false
            return false;
        }
        else {
            throw new Error("@textlint/feature-flag:Error: " + flagName + " is not defined.");
        }
    }
    var status = flagMap.get(flagName);
    if (typeof status !== "boolean") {
        throw new Error("flag should be boolean, but it is :" + status);
    }
    return status;
};
// == CORE Flags
// if run textlint --experimental, set experimental true by default
if (typeof process !== "undefined" && Array.isArray(process.argv) && process.argv.indexOf("--experimental") !== -1) {
    setFeature("core.experimental", true);
}
/**
 * Core flags
 * @type {{experimental, experimental, runningCLI, runningCLI, runningTester, runningTester}}
 */
export var coreFlags = {
    // Experimental
    get experimental() {
        return isFeatureEnabled("core.experimental", {
            loose: true
        });
    },
    set experimental(status) {
        setFeature("core.experimental", status);
    },
    // CLI
    get runningCLI() {
        return isFeatureEnabled("core.runningCLI", {
            loose: true
        });
    },
    set runningCLI(status) {
        setFeature("core.runningCLI", status);
    },
    // textlint-tester
    get runningTester() {
        return isFeatureEnabled("core.runningTester", {
            loose: true
        });
    },
    set runningTester(status) {
        setFeature("core.runningTester", status);
    }
};
/**
 * if current is not experimental, throw error message.
 * @param message
 */
export function throwWithoutExperimental(message) {
    if (coreFlags.runningCLI && !coreFlags.experimental) {
        throw Error(message);
    }
}
/**
 * if current is in testing, throw error message.
 * @param {string} message
 */
export function throwIfTesting(message) {
    if (coreFlags.runningTester) {
        throw Error(message);
    }
}
//# sourceMappingURL=index.js.map